package com.lantone.qc.kernel.structure.ai.process;


import com.alibaba.fastjson.JSONObject;
import com.lantone.qc.kernel.structure.ai.model.EntityEnum;
import com.lantone.qc.kernel.structure.ai.model.Lemma;
import com.lantone.qc.kernel.util.DiagEnhancer;
import com.lantone.qc.pub.model.entity.*;
import com.lantone.qc.pub.model.label.PresentLabel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.LinkedList;
import java.util.List;

/**
 * 现病史处理
 */
public class EntityProcessClinic extends EntityProcess {
    private Logger logger = LoggerFactory.getLogger(EntityProcessPast.class);

    public PresentLabel extractEntity(JSONObject aiOut) {
        PresentLabel presentLabel = new PresentLabel();
        try {
            //临床表现
            List<Lemma> clinicLemmas = createEntityTree(aiOut, EntityEnum.CLINICAL_FEATURE.toString());
            for (Lemma lemma : clinicLemmas) {
                Clinical clinical = new Clinical();
                clinical.setName(lemma.getText());

                List<PD> timestamp = new LinkedList<>();
                for (Lemma relationLemma : lemma.getRelationLemmas()) {
                    if (relationLemma.getProperty().equals(EntityEnum.NEGATIVE.toString())) {
                        clinical.setNegative(findNegative(lemma));
                    }
                    clinical.setModification(findT(lemma, new Modification(), EntityEnum.MODIFICATION.toString()));//修饰
                    clinical.setBodyPart(findT(lemma, new BodyPart(), EntityEnum.BODY.toString()));//部位
                    clinical.setTrend(findT(lemma, new Trend(), EntityEnum.TREND.toString()));//趋势
                    clinical.setCause(findT(lemma, new Cause(), EntityEnum.CAUSE.toString()));//诱因
                    clinical.setProperty(findT(lemma, new Property(), EntityEnum.PROPERTY.toString()));//性质
                    clinical.setDegree(findT(lemma, new Degree(), EntityEnum.DEGREE.toString()));//程度
                    clinical.setAggravate(findT(lemma, new Aggravate(), EntityEnum.AGGRAVATE.toString()));//加重因素
                    clinical.setRelief(findT(lemma, new Relief(), EntityEnum.RELIEF.toString()));//缓解因素
                    if (relationLemma.getProperty().equals(EntityEnum.TIME.toString())) {
                        String[] time_split = relationLemma.getText().split(",");
                        for (String time : time_split) {
                            PD pd = new PD();
                            String[] val_unit = new String[2];
                            if (time.trim().length() > 0) {
                                val_unit = extract_digit(time);
                            }
                            pd.setValue(val_unit[0]);
                            pd.setUnit(val_unit[1]);
                            pd.setName(time);
                            timestamp.add(pd);
                        }
                    }
                }
                presentLabel.add(presentLabel.getClinicals(), clinical);
            }
            //入院途径
            List<Lemma> beHospitalizedWayLemmas = createEntityTree(aiOut, EntityEnum.BEHOSPITALIZEDWAY.toString());
            for (Lemma lemma : beHospitalizedWayLemmas) {
                BeHospitalizedWay beHospitalizedWay = new BeHospitalizedWay();
                beHospitalizedWay.setName(lemma.getText());
                presentLabel.setBeHospitalizedWay(beHospitalizedWay);
            }
            //读取疾病信息
            List<Lemma> diagLemmas = createEntityTree(aiOut, EntityEnum.DIEASE.toString());
            for (Lemma lemma : diagLemmas) {
                Diag diag = DiagEnhancer.create(lemma.getText());
                if (lemma.isHaveChildren()) {
                    //阴性
                    diag.setNegative(findNegative(lemma));
                    //可能的
                    for (Lemma relationLemma : lemma.getRelationLemmas()) {
                        if (relationLemma.getProperty().equals(EntityEnum.POSSIBLE.toString())) {
                            Possible possible = new Possible();
                            possible.setName(relationLemma.getText());
                            diag.setPossible(possible);
                        }
                    }
                }
                presentLabel.add(presentLabel.getDiags(), diag);
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage(), e);
        }
        return presentLabel;
    }
}
